@@ -23,15 +23,16 @@ RDF::Generator::Void - Generate VoID descriptions based on data in an RDF model
 
 =head1 VERSION
 
-Version 0.01_16
+Version 0.05_1
 
-Note that this is an early alpha release. It has pretty limited
-functionality, and there may very significant changes in this module
+Note that this is a beta release. It has the core functionality in
+place to create a basic VoID description and what's there should be
+working well. Nevertheless significant changes in this module may be
 coming up really soon.
 
 =cut
 
-our $VERSION = '0.01_16';
+our $VERSION = '0.05_1';
 
 =head1 SYNOPSIS
 
@@ -80,6 +81,8 @@ has inmodel => (
 					 required => 1,
 					);
 
+# This is setting up the dataset_uri method, and make it possible to
+# create a resource of it from strings or URI objects.
 class_type 'URI';
 
 subtype 'DatasetURI',
@@ -98,6 +101,7 @@ has dataset_uri => (
 						  coerce   => 1,
 						 );
 
+# This will create a URN with a UUID by default
 sub _build_dataset_uri {
 	my ($self) = @_;
 	return iri sprintf('urn:uuid:%s', Data::UUID->new->create_str);
@@ -119,6 +123,9 @@ values should be a string that represents the URI of a vocabulary.
 
 =cut
 
+# All the following attributes have that in common that they
+# automatically the method names also specified in handles, to
+# manipulate and query the data.
 has vocabulary => (
 						 is       => 'rw',
 						 traits   => ['Array'],
@@ -203,6 +210,8 @@ is set.
 
 =cut
 
+# There should only be a single uriSpace per Dataset (but there may be
+# more for subsets), thus this is a simple scalar attribute.
 has urispace => (
 					  is        => 'rw',
 					  isa       => 'Str',
@@ -211,6 +220,17 @@ has urispace => (
 
 =head2 Running this stuff
 
+=head3 C<level>, C<has_level>
+
+Set the level of detail. 0 doesn't do any statistics or heuristics, 1
+has some statistics for the dataset as a whole. Setting no level will
+give everything.
+
+=cut
+
+has level => (is => 'rw', isa => 'Int', predicate => 'has_level');
+
+
 =head3 C<stats>, C<clear_stats>, C<has_stats>
 
 Method to compute a statistical summary for the data in the dataset,
@@ -219,6 +239,8 @@ clear the statistics and C<has_stats> will return true if exists.
 
 =cut
 
+# In practice, this method just calls the ::Stats class to do
+# everything.
 has stats => (
 				  is       => 'rw',
 				  isa      => 'RDF::Generator::Void::Stats',
@@ -234,18 +256,20 @@ sub _build_stats {
 }
 
 
-=head3 generate
+=head3 generate( [ $model ] )
 
-Returns the VoID as an RDF::Trine::Model.
+Returns the VoID as an RDF::Trine::Model. You may pass a model with
+statements as argument to this method. This model may then contain
+arbitrary RDF that will be added to the RDF model. If you do not send
+a model, one will be created for you.
 
 =cut
 
 sub generate {
 	my $self = shift;
+	my $void_model = shift || RDF::Trine::Model->temporary_model;
 
-	# Create a model for adding VoID description
-	local $self->{void_model} =
-	  my $void_model = RDF::Trine::Model->temporary_model;
+	local $self->{void_model} = $void_model;
 
 	# Start generating the actual VoID statements
 	$void_model->add_statement(statement(
@@ -254,20 +278,6 @@ sub generate {
 													 $void->Dataset,
 													));
 
-	if ($self->has_urispace) {
-		$void_model->add_statement(statement(
-														 $self->dataset_uri,
-														 $void->uriSpace,
-														 literal($self->urispace)
-														));
-		$self->_generate_counts($void->entities, $self->stats->entities);
-	}
-
-	$self->_generate_counts($void->distinctSubjects, $self->stats->subjects);
-	$self->_generate_counts($void->properties, $self->stats->properties);
-	$self->_generate_counts($void->distinctObjects, $self->stats->objects);
-
-
 	foreach my $endpoint ($self->all_endpoints) {
 		$void_model->add_statement(statement(
 														 $self->dataset_uri,
@@ -298,8 +308,31 @@ sub generate {
 													 $void->triples,
 													 literal($self->inmodel->size, undef, $xsd->integer),
 													));
+
+	if ($self->has_urispace) {
+		$void_model->add_statement(statement(
+														 $self->dataset_uri,
+														 $void->uriSpace,
+														 literal($self->urispace)
+														));
+		return $void_model if ($self->has_level && $self->level == 0);
+		$self->_generate_counts($void->entities, $self->stats->entities);
+	}
+
+	return $void_model if ($self->has_level && $self->level == 0);
+	$self->_generate_counts($void->distinctSubjects, $self->stats->subjects);
+	$self->_generate_counts($void->properties, $self->stats->properties);
+	$self->_generate_counts($void->distinctObjects, $self->stats->objects);
+
+
+
+
 	$self->_generate_most_common_vocabs($self->stats) if $self->has_stats;
-  
+
+	return $void_model if ($self->has_level && $self->level <= 1);
+
+	$self->_generate_propertypartitions;
+	$self->_generate_classpartitions;
 	return $void_model;
 }
 
@@ -313,6 +346,44 @@ sub _generate_counts {
 															  ));
 }
 
+sub _generate_propertypartitions {
+  my ($self) = @_;
+  return undef unless $self->has_stats;
+  my $properties = $self->stats->propertyPartitions;
+  while (my ($uri, $count) = each(%{$properties})) {
+    my $blank = blank();
+    $self->{void_model}->add_statement(statement(
+						 $self->dataset_uri,
+						 $void->propertyPartition,
+						 $blank));
+    $self->{void_model}->add_statement(statement($blank,
+						 $void->property,
+						 iri($uri)));
+    $self->{void_model}->add_statement(statement($blank,
+						 $void->triples,
+						 literal($count, undef, $xsd->integer)));
+  }
+}
+
+sub _generate_classpartitions {
+  my ($self) = @_;
+  return undef unless $self->has_stats;
+  my $classes = $self->stats->classPartitions;
+  while (my ($uri, $count) = each(%{$classes})) {
+    my $blank = blank();
+    $self->{void_model}->add_statement(statement(
+						 $self->dataset_uri,
+						 $void->classPartition,
+						 $blank));
+    $self->{void_model}->add_statement(statement($blank,
+						 $void->class,
+						 iri($uri)));
+    $self->{void_model}->add_statement(statement($blank,
+						 $void->triples,
+						 literal($count, undef, $xsd->integer)));
+  }
+}
+
 sub _generate_most_common_vocabs {
 	my ($self) = @_;
 
@@ -337,14 +408,11 @@ sub _generate_most_common_vocabs {
 
 Kjetil Kjernsmo C<< <kjetilk@cpan.org> >>
 Toby Inkster C<< <tobyink@cpan.org> >>
-Tope Omitola, C<< <tope.omitola at googlemail.com> >>
 
 =head1 TODO
 
 =over
 
-=item * Allow arbitrary RDF to be added to the VoID.
-
 =item * Larger test dataset for more extensive tests.
 
 =item * URI regexps support.
@@ -367,7 +435,7 @@ Tope Omitola, C<< <tope.omitola at googlemail.com> >>
 
 =item * Set URI space on partitions.
 
-=item * Conditional updates based on model ETags.
+=item * Save the description to files?
 
 =back
 
@@ -410,7 +478,7 @@ L<https://metacpan.org/module/RDF::Generator::Void>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2012 Tope Omitola, Kjetil Kjernsmo, Toby Inkster.
+Copyright 2012 Kjetil Kjernsmo, Toby Inkster.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
